# sb::App::Receipt
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Receipt;
use strict;
use sb::Language ();
use sb::Config ();
use sb::Data ();
use sb::Lock ();
use sb::Time ();
use sb::Net::Mailer ();
use sb::Template ();
use sb::Content ();
use sb::Text ();
use sb::Build ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.35';
# 0.35 [2008/11/24] modified _error_message to output an error message correctly
# 0.34 [2008/10/30] modified _sendmail to correct charcode and mail message
# 0.33 [2008/10/28] modified _sendmail to create a sb::Net::Mailer object correctly
# 0.32 [2007/12/03] modified _handle_receiver/_check_spam, fixed a bug in  _check_charset
# 0.31 [2007/08/16] renamed TemplateManager to Template, TemplateManager has been deprecated
# 0.30 [2007/08/14] added _check_mode/moved some functions from sb::App::Trackback
# 0.29 [2007/08/10] modified for Serene Bach 3.*
# 0.28 [2007/03/17] changed _message/_trackback to handle error message correctly
# 0.27 [2007/03/06] added new member variable for handling error
# 0.26 [2007/02/16] changed _output_trackback to correct encoding
# 0.25 [2007/02/09] changed _sendmail
# 0.24 [2007/02/09] added _handle_receiver
# 0.23 [2007/02/08] changed _message / _trackback to fix a bug
# 0.22 [2007/02/06] changed plugin entry point
# 0.21 [2007/01/05] changed _message to fix a bug
# 0.20 [2006/11/27] changed _output_message
# 0.19 [2006/11/09] changed bracket rule / ignored _check_charset temporarily
# 0.18 [2006/10/14] changed _check_charset
# 0.17 [2006/10/07] changed _trackback to check charset
# 0.16 [2006/09/30] changed _message and _trackback to check status correctly
# 0.15 [2006/09/10] changed _message and _trackback to check spam before sb::Data->add and handle status
# 0.14 [2005/10/19] changed _convert_message_text to conver linefeed as well, added _load_objs
# 0.13 [2005/10/18] changed _output_trackback to pass TemplateManager object to sb::Content
# 0.12 [2005/10/13] changed _trackback to ignore trachbacks which have no body
# 0.11 [2005/08/23] chnaged _message and _trackback to ban those when entry is closed
# 0.10 [2005/08/12] changed _check_ip to check ip more precisely
# 0.09 [2005/07/25] changed issue to call plugins
# 0.08 [2005/07/20] chnaged _sendmail to detitize text in mail body
# 0.07 [2005/07/16] changed _buikd_files to change the order of building files
# 0.06 [2005/07/08] changed _message to store cookie
# 0.05 [2005/07/08] fixed a bug in _check_departure to check caller correctly
# 0.04 [2005/06/29] fixed a bug in _sendmail, changed _trackback & _message to set tz
# 0.03 [2005/06/09] changed new to add 'caller' to instance value
# 0.02 [2005/06/07] changed _sendmail
# 0.01 [2005/06/01] build files after receiving
# 0.00 [2005/02/28] generated
# --------------------------------------
# // defines / static variables
sub PING_ERROR_DENIED  (){ "The ping is denied.\n" };
sub PING_ERROR_NO_URL  (){ "URL field is necessary.\n" };
sub PING_ERROR_NOENTRY (){ "There is no entry.\n" };
sub PING_ERROR_UNKNOWN (){ "Uknown error.\n" };
sub PING_RESPONSE_RSS  (){ 'request to output as rss' };
sub PING_RESPONSE_VIEW (){ 'request to output as view' };
sub TEMPLATE_MAIL_COM  (){ 'notification_comment.txt' };
sub TEMPLATE_MAIL_TB   (){ 'notification_trackback.txt' };
sub TEMPLATE_RSS_TB    (){ 'rss_for_trackback.rdf' };
sub CONTENT_TYPE       (){ 'text/xml' };
sub OUTPUT_CHARSET     (){ 'utf-8' };
sub COOKIE_CHARCODE    (){ 'utf8' };
sub COOKIE_EXPIRES     (){ 365 };
sub ERROR_TITLE        (){ ' | Notification' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'mode'   => undef, # mode => com or tb
		'cgi'    => undef, # cgi interface
		'id'     => undef, # id for entry
		'time'   => undef, # current time
		'error'  => undef, # error message
		'issuer' => undef, # issuer
		@_
	);
	$self->generate_accessor('error','issuer');
	return $self;
}
sub run
{
	my $class = shift;
	my $self = $class->new(
		'mode' => 'tb', # default mode is trackback
		@_
	);
	$self->_check_mode();
	$self->issue();
}
# --------------------------------------
# // public functions
sub issue
{
	my $self = shift;
	$self->issuer( (caller)[1] );
	die(sb->string('Unknown error')) if (! $self->mode() or $self->id() eq '');
	my $lock = sb::Lock->lock() or die(sb->string('Failed to lock'));
	my ($entry,$error) = ($self->{'mode'} eq 'tb') 
		? $self->_trackback() 
		: $self->_message();
	$self->error($error);
	if ($entry and ! $self->error())
	{
		sb::Data->update($entry);
		$self->_build_files($entry);
	}
	$lock->unlock;
	return ($self->{'mode'} eq 'tb') 
		? $self->_output_trackback() 
		: $self->_output_message();
}
sub mode
{
	my $self = shift;
	return $self->{'mode'};
}
sub id
{
	my $self = shift;
	return $self->{'id'};
}
# --------------------------------------
# // private functions - main
sub _trackback
{ # receiving a trackback, this function should return entry object of the target and error message
	my $self  = shift;
	my $entry = sb::Data->load('Entry','id'=>$self->id());
	return( $entry,PING_ERROR_NOENTRY ) if (!$entry);
	return( $entry,PING_RESPONSE_RSS ) if ($self->cgi->value('__mode') eq 'rss');
	my %tb = ( # buffering data for received trackback
		'eid'  => $entry->id,
		'stat' => 1,
		'date' => $self->time->data(),
		'body' => $self->_convert($self->cgi->value('excerpt')),
		'subj' => $self->_convert($self->cgi->value('title')),
		'name' => $self->_convert($self->cgi->value('blog_name')),
		'url'  => $self->_convert($self->cgi->value('url')),
		'tz'   => $self->conf->value('conf_timezone'),
		'host' => $self->cgi->remote_host()->{'addr'},
	);
	eval { # error check
		die(PING_ERROR_NO_URL) if ( $tb{'url'} eq '' or !$self->_check_url($tb{'url'}) );
		die(PING_ERROR_DENIED) if ( $tb{'body'} eq '' or $tb{'body'} =~ /^\s*$/ );
		die(PING_ERROR_DENIED) if ( $entry->atb == 0 );
		die(PING_ERROR_DENIED) if ( $entry->stat == 0 );
		die(PING_ERROR_DENIED) if ( $self->cgi->referer() ne '' );
		die(PING_ERROR_DENIED) if ( !$self->_check_ip($tb{'host'}) );
		die(PING_ERROR_DENIED) if ( !$self->_check_agent() );
	};
	return( $entry,$@ ) if ($@);
	return( $entry,PING_ERROR_DENIED ) if ($self->_handle_receiver($entry,'tb',\%tb));
	return( $entry,$self->error() ) if ($self->error() ne '');
	my $num = $entry->tb;
	my $new = $self->_check_redundant_tb(\%tb,$entry->id);
	$new = sb::Data->add('Trackback',%tb) if (!$new);
	if ($new)
	{
		$entry->tb( ++$num ) if ($new->stat == 1);
		$self->_sendmail('tb',$new,$entry);
		sb::Data->update($new);
	}
	return( $entry,'' );
}
sub _output_trackback
{
	my $self  = shift;
	my $error = $self->error();
	my $out = '';
	if ( $error eq PING_RESPONSE_RSS )
	{
		my $content = sb::Content->new('trackback' => {'rss' => \&_trackback_rss_callback});
		my $cms = sb::Template->new($self->load_template('file' => TEMPLATE_RSS_TB));
		$out = $content->output($cms,
			'mode' => 'ent',
			'id'   => $self->id(),
			'time' => $self->time->data(),
		);
	}
	else
	{
		if ($error)
		{
			chomp($error);
			$error = sprintf("<error>1</error>\n<message>%s</message>\n",$error);
		}
		else
		{
			$error = "<error>0</error>\n";
		}
		$out .= '<?xml version="1.0" encoding="' . $self->lang->charset() . '"?>' . "\n";
		$out .= sprintf("<response>\n%s</response>\n",$error);
	}
	$self->cgi->head(
		'type' => CONTENT_TYPE,
		'length' => length($out)
	);
	$self->output($out);
	return;
}
sub _message
{ # receiving a comment, this function should return entry object of the target and error message
	my $self  = shift;
	my $entry = sb::Data->load('Entry','id'=>$self->{'id'});
	return( $entry,"error_no_entry\n" ) if (!$entry);
	my %com = ( # buffering data for received comment
		'eid'  => $entry->id,
		'stat' => 1,
		'date' => $self->time->data(),
		'tz'   => $self->conf->value('conf_timezone'),
		'body' => $self->_convert_message($self->cgi->value('description')),
		'auth' => $self->_convert_message($self->cgi->value('name')),
		'mail' => $self->cgi->value('email'),
		'url'  => $self->cgi->value('url'),
		'host' => $self->cgi->remote_host()->{'addr'},
		'agnt' => $self->cgi->user_agent(),
		'icon' => $self->cgi->value('icon'),
	);
	$com{'icon'} = int($com{'icon'}) if ($com{'icon'} ne '');
	eval { # error check
		die("error_no_comment\n") if ( $com{'body'} eq '' );
		die("error_banned\n")     if ( $entry->acm == 0 );
		die("error_banned\n")     if ( !$self->_check_charset() );
		die("error_no_entry\n")   if ( $entry->stat == 0 );
		die("error_banned\n")     if ( !$self->_check_ip($com{'host'}) );
		die("error_doubled\n")    if ( $self->_check_redundant_com(\%com,$entry->id) );
	};
	return($entry,$@) if ($@);
	return($entry,"error_banned\n") if ($self->_handle_receiver($entry,'com',\%com));
	return($entry, $self->{'error'}) if ($self->{'error'} ne '');
	my $num = $entry->com;
	my $new = sb::Data->add('Message',%com);
	if ($new)
	{
		$entry->com( ++$num ) if ($new->stat == 1);
		$self->_sendmail('com',$new,$entry);
		sb::Data->update($new);
	}
	else
	{ # no new comment, it does not happen normally
		return($entry,"unknown_error\n");
	}
	if ($self->cgi->value('set_cookie'))
	{ # set cookies
		my %data = ();
		my $code = $self->lang->charcode_of($self->cgi->value('charset'));
		foreach my $key ( @{$self->conf->value('basic_cookie')} )
		{
			my $value = ($key ne 'checkid') ? $self->cgi->value($key) : $self->conf->value('conf_spamid');
			$data{$key} = ($code ne COOKIE_CHARCODE)
				? $self->lang->convert($value,COOKIE_CHARCODE,$code)
				: $value;
		}
		$self->cgi->set_cookie(
			'time'   => $self->time->data(),
			'name'   => $self->conf->value('basic_cooktag'),
			'expire' => COOKIE_EXPIRES,
			'data'   => \%data,
		);
	}
	return($entry,'') if ($new->stat == 1);
	return($entry,"error_wait_msg\n") if ($new->stat == 0);
	return($entry,"error_saved_as_closed\n");
}
sub _output_message
{
	my $self = shift;
	my $error = $self->error();
	my $out = '';
	if ($error)
	{
		chomp($error) if ($error =~ /\n$/);
		$out = ($error eq 'error_banned')
			? $self->{'cgi'}->head('status'=>'403 Forbidden')
			: $self->{'cgi'}->head('type'=>'text/html');
		$out .= $self->_error_message( sb->string($error) );
	}
	else
	{
		my $redirect = $self->_check_departure( sb::Data->load('Entry','id'=>$self->{'id'}) );
		$redirect .= '&com=0' if (index($self->{'caller'},'Mobile.pm') > -1);
		$out = $self->cgi->head('location'=>$redirect);
	}
	return($out);
}
sub _error_message
{
	my $self = shift;
	my $message = shift;
	my $cms = sb::Template->new( &sb::App::_error_template() );
	$cms->num(0);
	$cms->tag('sb_site_title'=>$sb::PRODUCT . ERROR_TITLE);
	$cms->tag('sb_error_title'=>sb->string('parts_error'));
	$cms->tag('sb_error'=>$message);
	$cms->tag('sb_site_top'=>sb::Config->get->value('conf_srv_base'));
	$self->common_parts($cms);
	return $cms->output;
}
sub _handle_receiver
{
	my $self  = shift;
	my $entry = shift;
	my $type  = shift;
	my $obj   = shift;
	my $conf = sb::Config->get;
	my @funcs = sb::Plugin->load_extra_module('receipt');
	push(@funcs,\&_check_spam); # add default handler
	my $acceptance = ($type eq 'tb') ? $entry->atb : $entry->acm;
	my %flag = (
		'black'=>undef,
		'white'=>undef,
		'close'=>undef
	);
	foreach my $func (@funcs)
	{
		eval {
			my $spam = &$func($self,$entry,$type,$obj);
			if ($spam > 0)
			{ # black
				$flag{'black'} = 1;
			}
			elsif ($spam < 0)
			{ # white
				$flag{'white'} = 1;
			}
			elsif ($obj->{'stat'} == -1)
			{ # gray and stat is set as 'closed'
				$flag{'close'} = 1;
			}
			$obj->{'stat'} = 1; # put stat back to 1
		};
	}
	if ($flag{'black'})
	{ # black - probably spam
		my $check = ($type eq 'com') 
			? $conf->value('conf_spamstat') 
			: $conf->value('conf_spamtbstat');
		return( 1 ) if (!$flag{'white'} and $check);
		$obj->{'stat'} = 0;
	}
	else
	{ # not black
		if ($acceptance == 2)
		{
			$obj->{'stat'} = 0;
		}
		elsif ($flag{'close'})
		{
			$obj->{'stat'} = -1;
		}
		else
		{
			$obj->{'stat'} = 1;
		}
	}
	return;
}
sub _sendmail
{
	my $self = shift;
	my ($mode,$obj,$entry) = @_;
	my $blog = sb::Data->load('Weblog','id'=>0);
	return if ($blog->smtp eq '' or $blog->stype eq '');
	my ($to,$from,$body) = ();
	my @to = ();
	my @users = sb::Data->load('User','cond'=>{'notice'=>1},'param' => 'notice');
	foreach my $user (@users)
	{
		next if (!$user->mail);
		$from = $user->mail if ($from eq '');
		push(@to,$user->mail);
	}
	$to = join(', ',@to);
	return if ($to eq '' or $from eq '');
	my $tag  = ($mode eq 'tb') ? 'trackback' : 'comment';
	my $file = ($mode eq 'tb') ? TEMPLATE_MAIL_TB : TEMPLATE_MAIL_COM;
	my $cms  = sb::Template->new($self->load_template('file' => $file));
	my $date = sb::Time->format(
		'time'=>$obj->date,
		'form'=>$self->conf->value('conf_msg_time'),
		'zone'=>$obj->tz,
		'lang'=>$self->conf->value('conf_time_lang'),
	);
	my $admin = $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_admn') . '?__mode=' . $tag;
	$cms->num(0);
	$cms->tag('notification_title'=>sb->string('The following ' . $tag . ' has been received.'));
	$cms->tag('entry_title'=>$entry->subj);
	$cms->tag('entry_parmalink'=>$entry->permalink);
	$cms->tag('receive_date'=>$date);
	$cms->tag('product_name'=>$sb::PRODUCT . ' ' . $sb::VERSION . ' ' . $sb::WEBPAGE);
	if ($mode eq 'tb')
	{
		$cms->tag('tb_blogname'=>$obj->name);
		$cms->tag('tb_title'=>$obj->subj);
		$cms->tag('tb_url'=>$obj->url);
		$cms->tag('tb_host'=>$obj->host);
		$cms->tag('tb_admin_link'=>$admin . '&bid=' . $obj->id);
		$cms->tag('tb_excerpt'=>sb::Text->new($obj->formated_body)->detitized());
		$cms->block('tb_blogname'=>1) if ($obj->name ne '');
		$cms->block('tb_title'=>1) if ($obj->subj ne '');
	}
	else
	{
		$cms->tag('com_name'=>$obj->auth);
		$cms->tag('com_mail'=>$obj->mail);
		$cms->tag('com_url'=>$obj->url);
		$cms->tag('com_host'=>$obj->host);
		$cms->tag('com_admin_link'=>$admin . '&mid=' . $obj->id);
		$cms->tag('com_body'=>sb::Text->new($obj->body)->detitized());
		$cms->block('com_name'=>1) if ($obj->auth ne '');
		$cms->block('com_mail'=>1) if ($obj->mail ne '');
		$cms->block('com_url'=>1) if ($obj->url ne '');
	}
	my $mailer = sb::Net::Mailer->new(
		'sender' => {
			'type'    => $blog->stype,
			'address' => $blog->smtp,
		},
		'charcode'  => $self->lang->mailcode,
	);
	$mailer->sendmail(
		'To'      => $to,
		'From'    => $from,
		'Subject' => sprintf('[%s]%s',$sb::PRODUCT,sb->string($tag . ' notification')),
		'Body'    => $cms->output,
	);
}
sub _check_mode
{
	my $self = shift;
	my $id = undef;
	if ($self->mode eq 'tb')
	{ # works as trackback receiver
		$id = $self->cgi->value('tb');
		if ($self->cgi->path_info() ne '')
		{
			$id = $self->cgi->path_info();
			$id =~ s/.*[:\/\\](.*)/$1/;
		}
	}
	else
	{ # works as comments receiver
		$id = $self->cgi->value('entry_id');
	}
	$id = undef if ($id !~ /^\d+$/);
	$self->{'id'} = $id;
}
# --------------------------------------
# // private functions - utilities
sub _convert
{
	my $self = shift;
	my $text = shift;
	my $lang = $self->lang;
	my $code = $lang->charcode_of($self->cgi->value('charset'));
	return $lang->convert($text,$lang->charcode,$code) if ($lang->charcode ne $code);
	return $text;
}
sub _convert_message
{
	my $self = shift;
	my $text = shift;
	my $ua   = $self->cgi->user_agent();
	my $output = '';
	# sanitizing text from mobile terminal
	#  => reference from http://specters.net/cgipon/labo/it_emoji.html
	if ($ua =~ /DoCoMo/ && $text =~ /[\xF8\xF9]/)
	{
		while (1)
		{
			if ($text =~ s/^[\xF8\xF9][\x40-\x7E\x80-\xFC]//)
			{
				$output .= '&#' . unpack('n', $&) . ';';
			}
			elsif ($text =~ s/^([\x81-\x9F\xE0-\xF7\xFA-\xFC][\x40-\x7E\x80-\xFC])+//)
			{
				$output .= $&;
			}
			elsif ($text =~ s/^.//)
			{
				$output .= $&;
			}
			else
			{
				last;
			}
		} # end of while(1)
	} # end of if ($ua =~ /DoCoMo/ && $text =~ /[\xF8\xF9]/)
	else
	{
		$output = $text;
	}
	# unify linefeed code
	$output =~ s/\x0D\x0A/\n/g;
	$output =~ tr/\x0D\x0A/\n\n/;
	while ($output =~ /\n$/)
	{
		$output =~ s/\n$//g;
	}
	return sb::Text->new($self->_convert($output))->entitized();
}
sub _check_redundant_tb
{
	my $self = shift;
	my $ref  = shift;
	my $eid  = shift;
	my @objs = sb::Data->load('Trackback','cond'=>{'eid'=>$eid});
	foreach my $obj ( @objs )
	{
		if ( $ref->{'url'} eq $obj->url )
		{
			$obj->date($ref->{'date'});
			$obj->body($ref->{'body'});
			$obj->subj($ref->{'subj'});
			$obj->name($ref->{'name'});
			$obj->host($ref->{'host'});
			return $obj;
		}
	}
	return;
}
sub _check_redundant_com
{
	my $self = shift;
	my $ref  = shift;
	my $eid  = shift;
	my @objs = sb::Data->load('Message','cond'=>{'eid'=>$eid});
	foreach my $obj ( @objs )
	{
		return 1 if ( $ref->{'auth'} eq $obj->auth and $ref->{'body'} eq $obj->body );
	}
	return;
}
sub _check_agent
{
	my $self = shift;
	my $ua   = $self->cgi->user_agent();
	my $check = 1;
	my @aDenied = ('Mozilla/');
	foreach my $list ( @aDenied )
	{
		$check = 0 if (index($ua,$list) == 0);
		last if (!$check);
	}
	return $check;
}
sub _check_charset
{
	my $self = shift;
	my $charset = $self->cgi->value('charset');
	return if ($charset eq ''); # charset must be set
	my $code  = $self->lang->charcode_of($charset);
	my $level = $self->conf->value('conf_spamlevel');
	return 1 if ($code eq 'utf8');
	if ($self->lang->language eq 'ja' and ($level == 3 or $level == 5))
	{
		return 1 if ($code eq 'euc' or $code eq 'jis' or $code eq 'sjis');
		return;
	}
	return 1;
}
sub _check_url
{
	my $self = shift;
	return( $_[0] =~ /s?https?:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+/ );
}
sub _check_ip
{
	my $self = shift;
	my $addr = shift;
	my $check = 1;
	my @ban_list = split("\n",$self->conf->value('conf_ip_ban'));
	foreach my $ban_ip ( @ban_list )
	{
		$check = 0 if ($addr =~ /^$ban_ip/);
		last if (!$check);
	}
	return $check;
}
sub _check_departure
{
	my $self  = shift;
	my $entry = shift;
	my $type = (index($self->issuer(),'Mobile.pm') > -1) 
	         ? 'Mobile' 
	         : $self->conf->value('conf_entry_archive');
	$type = 'None' if ($type eq 'Monthly');
	return $entry->permalink('type'=>$type);
}
sub _check_spam
{ # default receipt handler
	my $self = shift;
	my ($entry,$type,$obj) = @_;
	my $level = ($type eq 'com')
		? $self->conf->value('conf_spamlevel')
		: $self->conf->value('conf_spamtblevel');
	if ($level > 0)
	{
		if ($level < 3 and $type eq 'com')
		{ # level 1 and 2
			my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_cooktag'));
			return 1 if (index($self->cgi->referer(),$self->_check_departure($entry)) == -1); # level 1
			my $check_ref = $entry->permalink('type'=>'None');
			my $check_id = $cookie->{'checkid'};
			return 1 if (  $level > 1 and $check_id ne $self->conf->value('conf_spamid')); # level 2
		}
		elsif ($level == 3 and $self->lang->language eq 'ja' and $obj->{'body'} !~ m/[\x80-\xff]/)
		{ # level 3 [Japanese only]
			return 1;
		}
		elsif ($level >= 4)
		{ # level 4
			my $word = ($type eq 'com') 
				? $self->conf->value('conf_spamword') 
				: $self->conf->value('conf_spamtbword');
			foreach my $cond ( split("\n",$word) )
			{
				my ($field,$val) = split('=',$cond,2);
				my $check = $obj->{'body'};
				next if ($type eq 'com' and $field eq 'title');
				next if ($type eq 'tb' and $field eq 'mail');
				$check = $obj->{'mail'} if ($field eq 'mail');
				$check = $obj->{'subj'} if ($field eq 'title');
				$check = $obj->{($type eq 'tb') ? 'name' : 'auth'} if ($field eq 'name');
				$check = $obj->{'url'}  if ($field eq 'url');
				return 1 if (index($check,$val) > -1 or index(lc($check),lc($val)) > -1);
			}
			if ($level == 5 and $self->lang->language eq 'ja' and $obj->{'body'} !~ m/[\x80-\xff]/)
			{ # level 5 [Japanese only]
				return 1;
			}
		}
	}
	return; # passed default spam check
}
sub _trackback_rss_callback
{
	my $cms = shift;
	my $tb  = shift;
	my %var = @_;
	$cms->tag('trackback_title_only'=>$tb->subj);
	$cms->tag('trackback_url_only'=>$tb->url);
	return;
}
sub _build_files
{
	my $self = shift;
	my $entry = shift;
	return if (!$entry);
	my %cat  = sb::Data->load_as_hash('Category');
	my %user = sb::Data->load_as_hash('User');
	my $builder = sb::Build->new(
		'time'      => $self->time,
		'user'      => \%user,
		'cat'       => \%cat,
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%cat) ],
		'blog'      => sb::Data->load('Weblog','id'=>0),
	);
	if ($self->conf->value('conf_entry_archive') eq 'Individual')
	{
		$builder->build_javascript('recent_comment_list') if ($self->mode() eq 'com');
		$builder->build_javascript('recent_trackback_list') if ($self->mode() eq 'tb');
		$builder->build_entry( $entry ) if ($entry->stat != 0);
	}
	elsif ($self->conf->value('conf_entry_archive') eq 'Monthly')
	{
		my $month = sb::Time->format(
			'time'=>$entry->date,
			'form'=>'%Year%%Mon%',
			'zone'=>$entry->tz
		);
		$builder->build_monthly_archive( $month );
	}
	$builder->build_category_index( $entry->cat ) if ($entry->cat ne '');
	$builder->build_top_page();
	return 1;
}
1;
__END__
